# -*- coding: utf-8 -*-

import json

import psycopg2
from shapely.geometry import Polygon
import sys

from src.affected_space.affected_space import get_affected_spaced
from src.grid.check_grid import check_grid_upright

space_keys = [
    'floor_id',
    'space_id',
    'outline'
]

model_keys = [
    'id'
]

ispace_keys = [
    'source_id',
    'outline'
]


def get_data(sql):
    global connection, cursor
    record = []
    try:
        connection = psycopg2.connect(
            database='datacenter',
            user='postgres',
            password='123456',
            host='192.168.20.234',
            port='5432'
        )
        cursor = connection.cursor()
        cursor.execute(sql)
        record = cursor.fetchall()
    except (Exception, psycopg2.Error) as error:
        print("Error while connecting to PostgreSQL", error)
    finally:
        if connection:
            cursor.close()
            connection.close()
            print('PostgreSQL connection is closed')
    return record


def loads(x):
    x['location'] = json.loads(x['location'])
    return x


def loads_curve(x):
    x['curve'] = json.loads(x['curve'])
    return x

def transform_outline(obj):
    column_outline = 'outline'
    for space in obj:
        outline = space.get(column_outline)
        print(type(outline))
        outline_str = str(outline).replace('\'', '"')
        space[column_outline] = outline_str


if __name__ == '__main__':
    # points1 = [(0, 0), (0, 1), (1, 1), (1, 0)]
    # points2 = [(0, 0), (0.00, 0.5), (0, 1), (1, 1), (1, 0)]
    # poly1 = Polygon(points1)
    # poly2 = Polygon(points2)
    #
    # print(poly1.equals(poly2))
    # print(poly1.almost_equals(poly2, 0.99))
    # print(poly1.almost_equals(poly2, 0))
    # print(poly1.almost_equals(poly2, -10))
    # print(poly1.almost_equals(poly2, 2))
    # print(poly1.almost_equals(poly2, 0.0001))
    floor_id = '7893fe0c6cd511ea83548fd73aa29e85'
    floor_str = "SELECT id FROM public.floor where model_id = '{0}'".format(floor_id)

    floor_data = get_data(floor_str)

    floor_list = [dict(zip(model_keys, item)) for item in floor_data]

    if len(floor_list) == 0:
        print("no binding floor")
        sys.exit(0)
    floor_str = ""
    for fl in floor_list:
        floor_str = (floor_str + "'" + fl.get('id') + "',")
    floor_str = floor_str.strip(',')
    model_str = "select id from revit.model_file where model_floor_id = '{0}' and removed = false and status = 4 and version is not null order by version desc limit 2".format(floor_id)
    model_data = get_data(model_str)
    model_list = [dict(zip(model_keys, item)) for item in model_data]

    if len(model_list) != 2 :
        print("no prev model")
        sys.exit(0)

    space_str = "select rel.floor_id, rel.space_id, sp.outline from r_sp_in_fl rel left join zone_space_base sp on rel.space_id = sp.id where rel.floor_id in (" + floor_str + ")"
    space_data = get_data(space_str)
    space_list = [dict(zip(space_keys, item)) for item in space_data]

    if len(space_list) == 0:
        print("no space")
        sys.exit(0)

    new_ispace_str = "select source_id, outline from revit.space where model_id = '{0}'".format(model_list[0]['id'])
    prev_ispace_str = "select source_id, outline from revit.space where model_id = '" + model_list[1]['id'] + "'"

    new_ispace_data = get_data(new_ispace_str)
    prev_ispace_data = get_data(prev_ispace_str)

    new_ispace_list = [dict(zip(ispace_keys, item)) for item in new_ispace_data]
    prev_ispace_list = [dict(zip(ispace_keys, item)) for item in prev_ispace_data]


    transform_outline(space_list)
    transform_outline(new_ispace_list)
    transform_outline(prev_ispace_list)


    affected_space_ids = get_affected_spaced(space_list, new_ispace_list, prev_ispace_list)

    print(affected_space_ids)

    # involved_space = "select * from  where folder_id = " \
    #                      "'bbe510dbe26011e999b69b669ea08505' and status in (3, 31, 4) "
    # grid_sql = "select * from revit.grid where model_id = "
    # columns_data = get_data(involved_model_sql)
    #
    # model_list = [dict(zip(involved_model_keys, item)) for item in columns_data]
    # if len(model_list) < 2:
    #     sys.exit()
    # grid_data = dict()
    # for item in model_list:
    #     current_grid_sql = grid_sql + '\'{model_id}\''.format(model_id=item.get('fid'))
    #     single_model_grid = get_data(current_grid_sql)
    #     single_model_grid = [dict(zip(grid_keys, item)) for item in single_model_grid]
    #     grid_data[item.get('fid')] = single_model_grid
    #
    # print(check_grid_upright(model_list, grid_data))

